@@ -8,7 +8,7 @@
 #        .
 #          .
 #
-#	SCCS: @(#)Expression.pm	1.38 03/27/15 18:50:40
+#	SCCS: @(#)Expression.pm	1.40 03/29/15 22:18:16
 #
 # This module is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself. You must preserve this entire copyright
@@ -50,7 +50,7 @@ use vars qw/
 	$Version
 );
 
-our $VERSION = "1.38";
+our $VERSION = "1.40";
 
 # Fundamental to this is a tree of nodes.
 # Nodes are hashes with members:
@@ -68,51 +68,52 @@ our $VERSION = "1.38";
 # 2 values, depending on if it is the TopOfStack or JustRead operator - [TOS, JR]. See ':=' which right associates.
 # Just binary operators makes life easier as well.
 # Getting the precedence values right is a pain and for things like close paren, non obvious.
+# Far apart numbers makes adding new ones easier.
 
 my %OperPrec = (
-	'var'	=>	[24, 24],
-	'const'	=>	[24, 24],
-	'['	=>	[7, 23],
-	'++'	=>	[22, 22],
-	'--'	=>	[22, 22],
-	'M-'	=>	[20, 21],	# Monadic -
-	'M+'	=>	[20, 21],	# Monadic +
-	'M!'	=>	[20, 21],
-	'M~'	=>	[20, 21],
-	'**'	=>	[19, 19],
-	'*'	=>	[18, 18],
-	'/'	=>	[18, 18],
-	'%'	=>	[18, 18],
-	'+'	=>	[17, 17],
-	'-'	=>	[17, 17],
-	'.'	=>	[16, 16],
-	'>'	=>	[15, 15],
-	'<'	=>	[15, 15],
-	'>='	=>	[15, 15],
-	'<='	=>	[15, 15],
-	'=='	=>	[15, 15],
-	'!='	=>	[15, 15],
-	'<>'	=>	[15, 15],
-	'lt'	=>	[15, 15],
-	'gt'	=>	[15, 15],
-	'le'	=>	[15, 15],
-	'ge'	=>	[15, 15],
-	'eq'	=>	[15, 15],
-	'ne'	=>	[15, 15],
-	'&&'	=>	[14, 14],
-	'||'	=>	[13, 13],
-	':'	=>	[12, 12],
-	'?'	=>	[11, 11],
-	','	=>	[10, 10],
-	'('	=>	[9, 22],
-	')'	=>	[9, 9],	
-	'func'	=>	[21, 22],
-	']'	=>	[7, 7],
-	':='	=>	[5, 6],		# 6 to make := right assosc
-	'}'	=>	[4, 0],
-	'flow'	=>	[3, 4],
-	';'	=>	[2, 2],
-	'{'	=>	[1, 0],
+	'var'	=>	[240, 240],
+	'const'	=>	[240, 240],
+	'['	=>	[70, 230],
+	'++'	=>	[220, 220],
+	'--'	=>	[220, 220],
+	'M-'	=>	[200, 210],	# Monadic -
+	'M+'	=>	[200, 210],	# Monadic +
+	'M!'	=>	[200, 210],
+	'M~'	=>	[200, 210],
+	'**'	=>	[190, 190],
+	'*'	=>	[180, 180],
+	'/'	=>	[180, 180],
+	'%'	=>	[180, 180],
+	'+'	=>	[170, 170],
+	'-'	=>	[170, 170],
+	'.'	=>	[160, 160],
+	'>'	=>	[150, 150],
+	'<'	=>	[150, 150],
+	'>='	=>	[150, 150],
+	'<='	=>	[150, 150],
+	'=='	=>	[150, 150],
+	'!='	=>	[150, 150],
+	'<>'	=>	[150, 150],
+	'lt'	=>	[150, 150],
+	'gt'	=>	[150, 150],
+	'le'	=>	[150, 150],
+	'ge'	=>	[150, 150],
+	'eq'	=>	[150, 150],
+	'ne'	=>	[150, 150],
+	'&&'	=>	[140, 140],
+	'||'	=>	[130, 130],
+	':'	=>	[120, 120],
+	'?'	=>	[110, 110],
+	','	=>	[100, 101],		# Build list 1,2,3,4 as ,L[1]R[,L[2]R[,L[3]R[4]]]
+	'('	=>	[90, 220],
+	')'	=>	[90, 90],	
+	'func'	=>	[210, 220],
+	']'	=>	[70, 70],
+	':='	=>	[50, 60],		# 6 to make := right assosc
+	'}'	=>	[40, 00],
+	'flow'	=>	[30, 40],
+	';'	=>	[20, 20],
+	'{'	=>	[10, 0],
 	'EOF'	=>	[-50, -50],
 );
 # TOS, JR
@@ -141,7 +142,9 @@ my %MatchOp = (
 my %MatchOpClose = reverse %MatchOp; # Reverse lookup
 
 # Inbuilt functions
-my %InFuns = map { $_ => 1} qw/ int abs round split join printf mktime strftime localtime defined aindex /;
+my %InFuns = map { $_ => 1} qw/ abs aindex count defined int join localtime mktime pop printf push round shift split strftime strlen unshift /;
+# Inbuilt functions that must be given a L value
+my %InFunLV = map { $_ => 1} qw / defined pop push shift unshift /;
 
 # Default error output function
 sub PrintError {
@@ -305,8 +308,8 @@ sub ParseString {
 			$newt->{oper} = 'func';
 			$newt->{fname} = $1;
 			$operlast = 1;    # So that argument can be monadic
-		} # Match ${SomeNonWhiteCharsOrCurlies} or VarName or $VarName or $123 or $OneNonSpaceCharacter
-		elsif($expr =~ s/^\$\{([^\s{}]+)\}|^\$(\d+|[_a-zA-Z]\w*|[^\s])|^([_a-zA-Z]\w*)//) {
+		} # Match VarName or $VarName or $123
+		elsif($expr =~ s/^\$?([_a-z]\w*)//i) {
 			$newt->{oper} = 'var';
 			$newt->{name} = defined($1) ? $1 : defined($2) ? $2 : $3;
 			$operlast = 0;
@@ -486,6 +489,15 @@ sub CheckTreeInt {
 		$ok = 0;
 	}
 
+	if($tree->{oper} eq 'func') {
+		my $fname = $tree->{fname};
+		if(($fname eq 'pop' or $fname eq 'shift' or $fname eq 'push' or $fname eq 'unshift') and
+		   (!defined($tree->{right}->{oper}) or (($tree->{right}->{oper} ne 'var' and $tree->{right}->{oper} ne ',') and (!defined($tree->{right}->{left}->{oper}) or $tree->{right}->{left}->{oper} ne 'var')))) {
+			$self->{PrintErrFunc}("First argument to $fname must be a variable");
+			$ok = 0;
+		}
+	}
+
 	$ChkErrs = 1 unless($ok);
 	return $ok ? $tree : undef;
 }
@@ -588,21 +600,31 @@ sub EvalTree {
 	return $tree->{val}										if($oper eq 'const');
 	return $wantlv ? $tree->{name} : $self->{VarGetFun}($self, $tree->{name})			if($oper eq 'var');
 
-	# Recognise the 'defined' func specially - it needs a lvalue
-	return $self->{FuncEval}($self, $tree->{fname},
-				$self->EvalTree($tree->{right}, $tree->{fname} eq 'defined'))		if($oper eq 'func');
+	# Some functions need to be given a lvalue
+	return $self->{FuncEval}($self, $tree, $tree->{fname},
+				$self->EvalTree($tree->{right}, defined($InFunLV{$tree->{fname}})))	if($oper eq 'func');
 
 	if($oper eq '++' or $oper eq '--') {
 		my ($right, @right, @left, $index, $name);
 		# The variable is either a simple variable or an indexed array
 		if($tree->{right}->{oper} eq '[') {
 			$name = $tree->{right}->{left}->{name};
-			$index = $self->EvalTree($tree->{right}->{right}, 0);
-			unless($index =~ /^\d+$/) {
+			$index = 1;
+
+			@left = $self->EvalTree($tree->{right}->{left}, 0);
+
+			@right = $self->EvalTree($tree->{right}->{right}, 0);
+			$index = $right[-1];
+
+			unless($index =~ /^-?\d+$/) {
 				$self->{PrintErrFunc}("Array '%s' index is not integer '%s'", $name, $index);
 				return undef;
 			}
-			@left = $self->EvalTree($tree->{right}->{left}, 0);
+
+			$index += @left if($index < 0);	# Convert -ve index to a +ve one, will still be -ve if it was very -ve to start with
+
+			return undef if($index < 0 or $index > @left);	# Out of bounds
+
 			$right = $left[$index];
 			$name = "$name\[$index\]";
 		} else {
@@ -739,15 +761,18 @@ sub EvalTree {
 	# Array index
 	if($oper eq '[') {
 		return undef	# Check if the array member could exist; ie have index
-			if($right !~ /^\d+$/);
+			if($right !~ /^-?\d+$/);
 
-		return "$left\[$right]" # Return var[index] for assignment
+		my $index = $right[-1];
+		$index += @left if($index < 0);	# Convert -ve index to a +ve one
+
+		return "$left\[$index]" # Return var[index] for assignment
 			if($wantlv);
 
 		return undef	# Check if the array member exists
-			if($right[-1] > @left);
+			if($index < 0 || $index > @left);
 
-		return $left[$right];
+		return $left[$index];
 	}
 
 
@@ -828,7 +853,7 @@ sub EvalTree {
 
 # Evaluate a function:
 sub FuncValue {
-	my ($self, $fname, @arglist) = @_;
+	my ($self, $tree, $fname, @arglist) = @_;
 
 	# If there is a user supplied extra function evaluator, try that first:
 	my $res;
@@ -863,6 +888,30 @@ sub FuncValue {
 
 	return $self->{VarIsDefFun}($self, $last)		if($fname eq 'defined');
 
+	if($fname eq 'pop' or $fname eq 'shift') {
+		my @a = $self->{VarGetFun}($self, $arglist[0]);
+		my $p = $fname eq 'pop' ? pop(@a) : shift(@a);
+		$self->{VarSetFun}($self, $last, @a);
+
+		return $p;
+	}
+
+	if($fname eq 'push' or $fname eq 'unshift') {
+		# Evaluate right->right and push/unshift that
+		my $vn = shift @arglist;		# var name
+
+		my @vv = $self->{VarGetFun}($self, $vn);# var value
+		my @vp = $self->EvalTree($tree->{right}->{right}, 0); # var to push/unshift
+
+		$fname eq 'push' ? push(@vv, @vp) : unshift(@vv, @vp);
+		$self->{VarSetFun}($self, $vn, @vv);
+
+		return scalar @vv;
+	}
+
+	return length($last)					if($fname eq 'strlen');
+	return scalar @arglist					if($fname eq 'count');
+
 	# aindex(array, val) returns index (from 0) of val in array, -1 on error
 	if($fname eq 'aindex') {
 		my $val = $arglist[$#arglist];
@@ -889,11 +938,11 @@ sub new {
 					EmptyArray	=>	[()],
 					EmptyList	=>	[()],
 			)},
-		VarGetFun	=>	\&VarGetFun,		# Get a variable function
-		VarIsDefFun	=>	\&VarIsDefFun,		# Is a variable defined function
-		VarSetFun	=>	\&VarSetFun,		# Set an array variable function
-		VarSetScalar	=>	\&VarSetScalar,		# Set a scalar variable function
-		FuncEval	=>	\&FuncValue,		# Evaluate function
+		VarGetFun	=>	\&VarGetFun,		# Get a variable - function
+		VarIsDefFun	=>	\&VarIsDefFun,		# Is a variable defined - function
+		VarSetFun	=>	\&VarSetFun,		# Set an array variable - function
+		VarSetScalar	=>	\&VarSetScalar,		# Set a scalar variable - function
+		FuncEval	=>	\&FuncValue,		# Evaluate - function
 		AutoInit	=>	0,			# If true auto initialise variables
 		ExtraFuncEval	=>	undef,			# User supplied extra function evaluator function
 		RoundNegatives	=>	0,			# Round behaves differently with -ve numbers
@@ -984,7 +1033,7 @@ variable values.
 Error messages may be via a user provided function.
 This is not designed for high computation use.
 
-=head1 DESCRIPTION
+=head1 HOW TO USE
 
 An expression needs to be first compiled (parsed) and the resulting tree may be run (evaluated)
 many times.
@@ -1048,6 +1097,12 @@ Beware this could take a long time to fail: C<printf('%1000000s', 'foo')>
 
 Default: 0.
 
+=item StringMaxLength
+
+The longest that a string may be.
+
+Default: 1000.
+
 =item PrintErrFunc
 
 This is a printf style function that will be called in the event of an error,
@@ -1245,6 +1300,18 @@ The variable C<_TIME> is initialised to the current time
 Applies the C<VarIsDefFun> to the last argument.
 Ie returns 1 if the variable is defined (has been assigned a value), 0 if it has not.
 
+=item push, pop, shift, unshift
+
+Add/remove elements from an array - as in perl.
+
+=item strlen
+
+The length of a string.
+
+=item count
+
+The number of elements in an array.
+
 =item aindex
 
 Searches the arguments for the last argument and returns the index.
@@ -1258,19 +1325,16 @@ Eg the following will return 1:
 
 =head2 Variables
 
-Variables can 3 three forms, there is no difference in usage between any of them.
-A variable name is either alphanumeric (starting with alpha, underscore is deemed an alpha), numeric (with
-a leading C<$> or C<${>, or any non white space between C<{}> or one non
-white space after a C<$>:
+Variables can two forms, there is no difference in usage between any of them.
+A variable name is either alphanumeric (starting with alpha, underscore is deemed an alpha), or
+the same name with a leading C<$>. Both refer to the same variable.
 
 	Variable
 	_foo123
 	$Variable
-	${Variable}
-	$123
-	${123}
-	$#
-	${###=##}
+	$_foo123
+
+A previous version of this module allowed more syntaxes.
 
 =head2 Literals
 
@@ -1351,7 +1415,7 @@ If there are not enough values the last ones are unchanged.
 
 You may use C<[]> to numerically index into arrays to obtain and set scalar values.
 Arrays cannot contain other arrays.
-Array indexes start with C<0>.
+Array indexes start with C<0>. Negative indicies index from the end of the array, thus -1 is the last element.
 
 	a := (20,21,22); a[1] + a[2]
 	a := (20,21,22); a[1] := 9; ++a[k + j]
@@ -1446,7 +1510,7 @@ Alain D D Williams <addw@phcomp.co.uk>
 
 =head2 Copyright and Version
 
-Version "1.38", this is available as: $Math::Expression::Version.
+Version "1.40", this is available as: $Math::Expression::Version.
 
 Copyright (c) 2003, 2015 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
